clc
clear
close
tic

% variables
lambda = 633.0e-09;
w0     = 1.0e-04;

LX     = 0.0625e-02;
LY     = 0.0625e-02;
z      = 0.0;
dz     = 0.001;
zmax   = 0.3;
NX     = 150;


% derived parameters
k0   = 2*pi/lambda;
dx   = LX/NX;

% transverse wavenumber
angle = 1.0;
kx    = k0*angle/360.0*2.0*pi;

% coordinates
index_x = 1:NX;
cx = (((index_x-1) -NX/2+0.5)*dx);

% initial amplitude
aux =  GaussianBeam1D(cx,0.0,w0,k0,0.0,-kx);
auy =  GaussianBeam1D(cx,0.0,w0,k0,0.0,-kx);

% PEC BC:
aux(1)   = 0.0;
aux(end) = 0.0;
auy(1)   = 0.0;
auy(end) = 0.0;

% 2D Gauss = 1Dx1D
E0  = auy'*aux; %'

% initailize PML %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

AA = ones(NX,1);
BB = zeros(NX,1);

ww = 1.0/(LX/4.0);

n0 = floor(0.15*NX);
n0 = NX-n0+1;
for x = n0:NX
    AA(x) = -1.0/(1i - 4.0*(ww*(cx(x) - cx(n0)))^3)^2.0;
    BB(x) = -6.0*1i*ww^3*(cx(x)-cx(n0))^2.0/(k0*(1i - 4.0*(ww*(cx(x) - cx(n0)))^3)^3);
end

n0 = floor(0.15*NX);
for x = 1:n0
    AA(x) =  -1.0/(1i + 4.0*(ww*(cx(x) - cx(n0)))^3)^2.0;
    BB(x) =  +6.0*1i*ww^3*(cx(x)-cx(n0))^2.0/(k0*(1i + 4.0*(ww*(cx(x) - cx(n0)))^3)^3);
end

E1 = reshape(E0,NX*NX,1);

tic;
 
while (z < zmax)
    % evolve amplitude
    zfin = z+dz;
    
    [zs Es] = PropagateV(E1,NX,dx,k0,z,dz,zfin,AA,BB);
    E1 = Es(end,:);
    
    z = z+dz;

imagesc(real(reshape(E1,NX,NX))); colorbar;

pause(0.01);

end

toc
