function [zsp enews] =  PropagateV(Ain,nx,dx,k0,zstart,dz,zfin,AA,BB)

% 2D square lattice propagator
% Uses Method of Lines
% PEC boundary

n2 = 1.0e-23;
IU = 3.0e+16;
ideltax = 1i/(2*k0*dx^2);
ideltay = ideltax;
inonl   = 1i*k0*n2*IU;

options = odeset('AbsTol', 1.0e-04, 'RelTol',1.0e-03);
[zsp enews] = ode23(@rhs2,zstart:dz:zfin,Ain);


    function da = rhs2(z,am)

        % total length af the input array - it is a column vector
        len = nx*nx;
        
        % allocate space for output
        da = zeros(len,1);


        % column operations:
        for y=2:nx-1
          coll = [1:1:nx] + (y-2)*nx; 
          colc = [1:1:nx] + (y-1)*nx; 
          colr = [1:1:nx] + (y-0)*nx; 
          da(colc) =  da(colc) + ideltax*(am(coll) -2.0*am(colc) +am(colr))*AA(y) +BB(y)*(am(colr) - am(coll))/(2.0*dx);
        end;

        
        % row operations:
        for x=2:nx-1
          rowu = [x-1:nx:len];
          rowc = [x+0:nx:len];
          rowd = [x+1:nx:len];
          da(rowc) =  da(rowc) + ideltax*(am(rowu) -2.0*am(rowc) +am(rowd))*AA(x) +BB(x)*(am(rowd) - am(rowu))/(2.0*dx);
        end;
        
        

        %zero boundaries
        %da([nx:nx:len])    = 0;
        %da([1:nx:len])     = 0;
        %da([1:1:nx])       = 0;
        %da([len-nx:1:len]) = 0;
                
        % nonlinear term
        % da = da + inonl*am.*abs(am).^2;
    end
end

